"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _write = _interopRequireDefault(require("../byte/write"));

var _read = _interopRequireDefault(require("./read"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 *	Modifies the buffer's bits to equal newBits starting at bitOffset.
 *
 *	@example
 *	modifyBuffer(buffer, [0,0,1,0], 0) => buffer was modified
 *
 *	@param buffer {Buffer} the buffer to modify
 *	@param bits {Array} the bits to insert
 *	@param offset {Number} where to start (in bits)
 *	@returns undefined
 */
var _default = function _default(buffer, bits) {
  var offset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  var start = Math.floor(offset / 8);
  var end = Math.ceil((offset + bits.length) / 8);
  var subBuffer = buffer.slice(start, end);
  var byteData = (0, _read.default)(subBuffer);
  var subOffset = offset % 8;

  for (var i = 0; i < bits.length; i++) {
    byteData[subOffset++] = bits[i];
  }

  var length = end - start;

  for (var _i = 0; _i < length; _i++) {
    subBuffer[_i] = (0, _write.default)(byteData.slice(_i * 8, (_i + 1) * 8));
  }
};

exports.default = _default;