"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.nand = exports.and = exports.xnor = exports.xor = exports.nor = exports.or = exports.not = void 0;

var not = function not(buffer) {
  return applyOperation('not', buffer, buffer, false);
};

exports.not = not;

var or = function or(buffer, otherBuffer, loop) {
  return applyOperation('or', buffer, otherBuffer, loop);
};

exports.or = or;

var nor = function nor(buffer, otherBuffer, loop) {
  return applyOperation('nor', buffer, otherBuffer, loop);
};

exports.nor = nor;

var xor = function xor(buffer, otherBuffer, loop) {
  return applyOperation('xor', buffer, otherBuffer, loop);
};

exports.xor = xor;

var xnor = function xnor(buffer, otherBuffer, loop) {
  return applyOperation('xnor', buffer, otherBuffer, loop);
};

exports.xnor = xnor;

var and = function and(buffer, otherBuffer, loop) {
  return applyOperation('and', buffer, otherBuffer, loop);
};

exports.and = and;

var nand = function nand(buffer, otherBuffer, loop) {
  return applyOperation('nand', buffer, otherBuffer, loop);
};
/**
 *	Helper method, contains the logic for the bitwise buffer operations in this script.
 *
 *	@example
 *	applyOperation('nand', buffer, otherBuffer);
 *
 *	@param type {String} the type of the operation
 *	@param buffer1 {Buffer} first buffer
 *	@param buffer2 {Buffer} second buffer
 *	@param isLooping {Boolean} loop through first buffer
 *	@return {Array}
 */


exports.nand = nand;

var applyOperation = function applyOperation(type, buffer1, buffer2) {
  var isLooping = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
  var length = isLooping ? buffer2.length : buffer1.length;
  var result = new Buffer(length);
  result.fill(0x00);

  for (var i = 0; i < length; i++) {
    var j = isLooping ? i % buffer1.length : i;

    switch (type) {
      case 'not':
        result[i] = ~buffer1[i];
        break;

      case 'or':
        result[i] = buffer1[j] | buffer2[i];
        break;

      case 'nor':
        result[i] = ~(buffer1[j] | buffer2[i]);
        break;

      case 'xor':
        result[i] = buffer1[j] ^ buffer2[i];
        break;

      case 'xnor':
        result[i] = ~(buffer1[j] ^ buffer2[i]);
        break;

      case 'and':
        result[i] = buffer1[j] & buffer2[i];
        break;

      case 'nand':
        result[i] = ~(buffer1[j] & buffer2[i]);
        break;
    }
  }

  return result;
};

var _default = {
  and: and,
  nand: nand,
  nor: nor,
  not: not,
  or: or,
  xnor: xnor,
  xor: xor
};
exports.default = _default;