"use strict";

var _ = _interopRequireDefault(require("../"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('buffer bitwise operations', function () {
  var bufferA, bufferB, bufferC;
  beforeEach(function () {
    bufferA = _.default.buffer.create(_.default.string.toBits('0110 1100 0110 1000 1110 0001 0010 1100'));
    bufferB = _.default.buffer.create(_.default.string.toBits('1011 0101 1001 1110 0001 1011 0100 0011'));
    bufferC = _.default.buffer.create(_.default.string.toBits('1010 1010'));
  });
  test('NOT', function () {
    expect(_.default.buffer.read(_.default.buffer.not(bufferA)).join()).toBe(_.default.string.toBits('1001 0011 1001 0111 0001 1110 1101 0011').join());
  });
  describe('OR', function () {
    test('equal size', function () {
      expect(_.default.buffer.read(_.default.buffer.or(bufferA, bufferB)).join()).toBe(_.default.string.toBits('1111 1101 1111 1110 1111 1011 0110 1111').join());
    });
    test('looping', function () {
      expect(_.default.buffer.read(_.default.buffer.or(bufferC, bufferA, true)).join()).toBe(_.default.string.toBits('1110 1110 1110 1010 1110 1011 1010 1110').join());
    });
  });
  describe('NOR', function () {
    test('equal size', function () {
      expect(_.default.buffer.read(_.default.buffer.nor(bufferA, bufferB)).join()).toBe(_.default.string.toBits('0000 0010 0000 0001 0000 0100 1001 0000').join());
    });
    test('looping', function () {
      expect(_.default.buffer.read(_.default.buffer.nor(bufferC, bufferA, true)).join()).toBe(_.default.string.toBits('0001 0001 0001 0101 0001 0100 0101 0001').join());
    });
  });
  describe('XOR', function () {
    test('equal size', function () {
      expect(_.default.buffer.read(_.default.buffer.xor(bufferA, bufferB)).join()).toBe(_.default.string.toBits('1101 1001 1111 0110 1111 1010 0110 1111').join());
    });
    test('looping', function () {
      expect(_.default.buffer.read(_.default.buffer.xor(bufferC, bufferA, true)).join()).toBe(_.default.string.toBits('1100 0110 1100 0010 0100 1011 1000 0110').join());
    });
  });
  describe('XNOR', function () {
    test('equal size', function () {
      expect(_.default.buffer.read(_.default.buffer.xnor(bufferA, bufferB)).join()).toBe(_.default.string.toBits('0010 0110 0000 1001 0000 0101 1001 0000').join());
    });
    test('looping', function () {
      expect(_.default.buffer.read(_.default.buffer.xnor(bufferC, bufferA, true)).join()).toBe(_.default.string.toBits('0011 1001 0011 1101 1011 0100 0111 1001').join());
    });
  });
  describe('AND', function () {
    test('equal size', function () {
      expect(_.default.buffer.read(_.default.buffer.and(bufferA, bufferB)).join()).toBe(_.default.string.toBits('0010 0100 0000 1000 0000 0001 0000 0000').join());
    });
    test('looping', function () {
      expect(_.default.buffer.read(_.default.buffer.and(bufferC, bufferA, true)).join()).toBe(_.default.string.toBits('0010 1000 0010 1000 1010 0000 0010 1000').join());
    });
  });
  describe('NAND', function () {
    test('equal size', function () {
      expect(_.default.buffer.read(_.default.buffer.nand(bufferA, bufferB)).join()).toBe(_.default.string.toBits('1101 1011 1111 0111 1111 1110 1111 1111').join());
    });
    test('looping', function () {
      expect(_.default.buffer.read(_.default.buffer.nand(bufferC, bufferA, true)).join()).toBe(_.default.string.toBits('1101 0111 1101 0111 0101 1111 1101 0111').join());
    });
  });
});